/*
 * Copyright (C) 2002 Ilya Volynets, Total Knowledge
 *
 * This program is free software. Free means freedom.
 * You are hereby granted the right to use this program
 * in any way you want as long as following conditions are met:
 * 1. All source distributions with code unmodified will also retain this
 *    notice intact.
 * 2. All binary distributions will be accompanied by text file clearly
 *    attributing authorship to Ilya Volynets and sopnsorship to
 *    Total Knowledge using following wording:
 *    "This binary is built from sources developed by Ilya Volynets.
 *    Development was kindly sponsored by Total Knowledge"
 * 3. All modified versions, if distributed, will clearly indicate
 *    that they are not original works and that they were derived from
 *    work developed by Ilya Volynets and sponsored by Total Knowledge
 */

#include "ssl++"

CSSLConnection::CSSLConnection(CSSLConnection::EType type, const std::string& connString,const std::string& cert_file, const std::string& key_file):
    type(type)
{
    int err;
    connection=SSL_new(ctx);
    valid=(bool)connection;
    if(!valid)return;
    if(!cert_file.empty())
	if(SSL_use_certificate_file(connection,cert_file.c_str(),SSL_FILETYPE_PEM)<=0) {
	    checkErrors("SSL_use_certificate_file");
	    valid=false;
	    return;
	}
    if(!key_file.empty())
	if(SSL_use_RSAPrivateKey_file(connection,key_file.c_str(),SSL_FILETYPE_PEM)<=0) {
	    checkErrors("SSL_use_RSAPrivateKey_file");
	    valid=false;
	    return;
	}
    if(!key_file.empty()&&!cert_file.empty())
	if(SSL_check_private_key(connection)<=0) {
	    checkErrors("SSL_check_private_key");
	    valid=false;
	    return;
	}
    if (SSL_set_cipher_list(connection,"ALL" )<=0) {
	checkErrors("SSL_set_cipher_list");
	valid=false;
	return;
    }

    if(type==CLIENT)
	valid=cliConnect(connString);
    else if (type==SERVER)
	valid=srvConnect(connString);
}

CSSLConnection::CSSLConnection(BIO* bio,const std::string& cert_file, const std::string& key_file):
    type(CSSLConnection::SERVER),bio(bio)
{
    int err;
    connection=SSL_new(ctx);
    valid=(bool)connection;
    if(!valid)return;
    if(!cert_file.empty())
	if(SSL_use_certificate_file(connection,cert_file.c_str(),SSL_FILETYPE_PEM)<=0) {
	    checkErrors("SSL_use_certificate_file");
	    valid=false;
	    return;
	}
    if(!key_file.empty())
	if(SSL_use_RSAPrivateKey_file(connection,key_file.c_str(),SSL_FILETYPE_PEM)<=0) {
	    checkErrors("SSL_use_RSAPrivateKey_file");
	    valid=false;
	    return;
	}
    if(!key_file.empty()&&!cert_file.empty())
	if(SSL_check_private_key(connection)<=0) {
	    checkErrors("SSL_check_private_key");
	    valid=false;
	    return;
	}
    if (SSL_set_cipher_list(connection,"ALL" )<=0) {
	checkErrors("SSL_set_cipher_list");
	valid=false;
	return;
    }

    valid=srvConnect("");
}
CSSLConnection::~CSSLConnection()
{
    SSL_free(connection);
}
bool CSSLConnection::srvConnect(const std::string& conn)
{
    if(!bio) {
	bio=BIO_new_accept((char*)conn.c_str());
        if(!bio) {
	    return false;
	}
	BIO_set_bind_mode(bio,BIO_BIND_REUSEADDR);
    }
    SSL_set_bio(connection,bio,bio);
    SSL_set_accept_state(connection);
    if(SSL_accept(connection)<=0) {
	checkErrors("SSL_accept");
	return false;
    }
    return true;
}

bool CSSLConnection::cliConnect(const std::string& conn)
{
    bio=BIO_new_connect((char*)conn.c_str());
    if(BIO_do_connect(bio)<=0) {
	valid=false;
	return false;
    }
    SSL_set_bio(connection,bio,bio);
    SSL_set_connect_state(connection);
    if(SSL_connect(connection)<=0) {
	checkErrors("SSL_connect");
	return false;
    }
    return true;
}


CSSLConnection& CSSLConnection::operator<<(const std::string& buf)
{
    SSL_write(connection,buf.c_str(),buf.length());
}

/**
 * Reads all the way to the end of file?
 * Or would it be better to just read available data?
 */
CSSLConnection& CSSLConnection::operator>>(std::string& buf)
{
#define READ_CHUNK_SIZE 1024
    char str[READ_CHUNK_SIZE];
    int ret;
    buf="";
    while((ret=SSL_read(connection,str,sizeof(str)-1))>0){
	str[ret]=0;
	buf+=str;
    }
    return *this;
}

