/*
 * Copyright (C) 2002 Ilya Volynets, Total Knowledge
 *
 * This program is free software. Free means freedom.
 * You are hereby granted the right to use this program
 * in any way you want as long as following conditions are met:
 * 1. All source distributions with code unmodified will also retain this
 *    notice intact.
 * 2. All binary distributions will be accompanied by text file clearly
 *    attributing authorship to Ilya Volynets and sopnsorship to
 *    Total Knowledge using following wording:
 *    "This binary is built from sources developed by Ilya Volynets.
 *    Development was kindly sponsored by Total Knowledge"
 * 3. All modified versions, if distributed, will clearly indicate
 *    that they are not original works and that they were derived from
 *    work developed by Ilya Volynets and sponsored by Total Knowledge
 */


#include "ssl++"

CSSLServer::CSSLServer(const std::string& connString, const std::string& certfile,const std::string& keyfile):
    certfile(certfile),keyfile(keyfile)
{
    accept=BIO_new_accept((char*)connString.c_str());
    if(!accept) {
	valid=false;
	return;
    }
    BIO_set_bind_mode(accept,BIO_BIND_REUSEADDR);
    if(BIO_do_accept(accept)<=0) {
	valid=false;
	checkErrors("BIO_do_accept(binding to socket)");
	return;
    }
}

CSSLServer::~CSSLServer()
{
    BIO_free_all(accept);
}

CSSLConnection* CSSLServer::nextClient()
{
    BIO* connect;
    if(!BIO_do_accept(accept)){
	checkErrors("BIO_do_accept (exteblishing connection)");
	return NULL;
    }
    connect=BIO_pop(accept);
    CSSLConnection *ret=new CSSLConnection(connect,certfile,keyfile);
    if (!*ret) {
	delete ret;
	return NULL;
    }
    return ret;
}
